"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.startControlServer = void 0;
const controller_1 = require("./controller");
const node_manager_1 = require("./node-manager");
const server_endpoint_1 = require("./ws/server-endpoint");
const ws_opcua_server_1 = require("./ws/ws-opcua-server");
function construct_control_address_space(controlServer) {
    const controller = new controller_1.Controller(controlServer, 1);
    const nodeManager = new node_manager_1.NodeManager(controlServer, 1);
    controller.on("test-server-started", (testServer) => (nodeManager.testServer = testServer));
    controller.on("test-server-stopped", (testServer) => (nodeManager.testServer = undefined));
    return controller;
    // (namespace as any).dispose();
    // declare a new object
    // _"add a new object into the objects folder"
    // add some variables
    // _"add some variables"
}
function startControlServer() {
    return __awaiter(this, void 0, void 0, function* () {
        const controlServer = new ws_opcua_server_1.WsOPCUAServer({
            port: 4840,
            alternateEndpoints: [
                {
                    transportType: server_endpoint_1.TransportType.WEBSOCKET,
                    port: 4444,
                },
            ],
            resourcePath: "",
            buildInfo: {
                productName: "wsopcua-control-server",
                buildNumber: "1",
                buildDate: new Date(Date.now()),
            },
            serverInfo: {
                applicationUri: "wsopcua-control-server",
            },
            serverCapabilities: {
                minSupportedSampleRate: 10,
            },
        });
        yield controlServer.initialize();
        console.log("control server initialized");
        const controller = construct_control_address_space(controlServer);
        controlServer.start(function (err) {
            if (err) {
                console.log("failed to start server !");
                console.log(err);
            }
            else {
                console.log("Server is now listening ... ( press CTRL+C to stop)");
                console.log("port ", controlServer.endpoints[0].port);
            }
        });
        return controller;
    });
}
exports.startControlServer = startControlServer;
//# sourceMappingURL=control-server.js.map