"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Controller = void 0;
const node_opcua_1 = require("node-opcua");
const load_nodeset2_1 = require("node-opcua-address-space/dist/source/loader/load_nodeset2");
const node_opcua_nodesets_1 = require("node-opcua-nodesets");
const test_server_1 = require("./test-server");
const events_1 = require("events");
/**
 * emits following events:
 * 'test-server-started' : WsOPCUAServer
 * 'test-server-stopped': WsOPCUAServer
 */
class Controller extends events_1.EventEmitter {
    constructor(controlServer, namespace) {
        super();
        this.controlServer = controlServer;
        this.namespace = namespace;
        this.nodesetMap = new Map([
            ["http://opcfoundation.org/UA/DI/", node_opcua_nodesets_1.nodesets.di],
        ]);
        this.init();
    }
    init() {
        var _a, _b;
        const ns = (_a = this.controlServer.engine.addressSpace) === null || _a === void 0 ? void 0 : _a.getNamespace(this.namespace);
        const obj = ns === null || ns === void 0 ? void 0 : ns.addObject({
            organizedBy: (_b = this.controlServer.engine.addressSpace) === null || _b === void 0 ? void 0 : _b.rootFolder.objects,
            browseName: "Controller",
            nodeId: node_opcua_1.coerceNodeId("s=Controller", this.namespace),
        });
        // expose supported test namespaces
        const varSupportedTestNamespaces = ns === null || ns === void 0 ? void 0 : ns.addVariable({
            browseName: "supportedTestNamespaces",
            nodeId: node_opcua_1.coerceNodeId("s=Controller.supportedTestNamespaces", 1),
            accessLevel: node_opcua_1.AccessLevelFlag.CurrentRead,
            dataType: node_opcua_1.DataType.String,
            valueRank: 1,
            value: new node_opcua_1.Variant({
                value: Array.from(this.nodesetMap.keys()),
                dataType: node_opcua_1.DataType.String,
                arrayType: node_opcua_1.VariantArrayType.Array,
            }),
            propertyOf: obj,
        });
        // start test server method
        const methodStartTestServer = ns === null || ns === void 0 ? void 0 : ns.addMethod(obj, {
            browseName: "startTestServer",
            nodeId: node_opcua_1.coerceNodeId("s=Controller.startTestServer", 1),
            inputArguments: [
                { name: "namespaces", dataType: node_opcua_1.DataType.String, valueRank: 1 },
            ],
        });
        methodStartTestServer === null || methodStartTestServer === void 0 ? void 0 : methodStartTestServer.bindMethod((inputArguments, _context, callback) => __awaiter(this, void 0, void 0, function* () {
            if (inputArguments[0].value.some((value) => !this.nodesetMap.has(value))) {
                // unknown namespace
                callback(new Error("namespace must be one of: " + this.nodesetMap.keys()), { statusCode: node_opcua_1.StatusCodes.BadInvalidArgument });
            }
            else {
                const nsFiles = inputArguments[0].value.map((namespace) => this.nodesetMap.get(namespace));
                yield this.startTestServer(nsFiles);
                callback(null, { statusCode: node_opcua_1.StatusCodes.Good });
            }
        }));
        // stop test server method
        const methodStopTestServer = ns === null || ns === void 0 ? void 0 : ns.addMethod(obj, {
            browseName: "stopTestServer",
            nodeId: node_opcua_1.coerceNodeId("s=Controller.stopTestServer", 1),
        });
        methodStopTestServer === null || methodStopTestServer === void 0 ? void 0 : methodStopTestServer.bindMethod((inputArguments, _context, callback) => __awaiter(this, void 0, void 0, function* () {
            yield this.stopTestServer();
            console.log("test server shut down");
            callback(null, { statusCode: node_opcua_1.StatusCodes.Good });
        }));
        // reload namespace method
        const methodReloadNamespace = ns === null || ns === void 0 ? void 0 : ns.addMethod(obj, {
            browseName: "reloadNamespace",
            nodeId: node_opcua_1.coerceNodeId("s=Controller.reloadNamespace", 1),
            inputArguments: [{ name: "namespaceUri", dataType: node_opcua_1.DataType.String }],
        });
        methodReloadNamespace === null || methodReloadNamespace === void 0 ? void 0 : methodReloadNamespace.bindMethod((inputArguments, _context, callback) => {
            if (inputArguments.length !== 1) {
                throw new Error("invalid argument count");
            }
            this.reloadNamespace(inputArguments[0].value);
            callback(null, { statusCode: node_opcua_1.StatusCodes.Good });
        });
    }
    startTestServer(namespaces) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.testServer) {
                this.testServer = yield test_server_1.startTestServer(namespaces);
            }
            this.emit("test-server-started", this.testServer);
        });
    }
    stopTestServer() {
        return __awaiter(this, void 0, void 0, function* () {
            const testServer = this.testServer;
            this.testServer = undefined;
            yield (testServer === null || testServer === void 0 ? void 0 : testServer.shutdown());
            this.emit("test-server-stopped", this.testServer);
        });
    }
    reloadNamespace(namespaceUri) {
        const index = this.deleteNamespace(namespaceUri);
        if (index >= 0) {
            this.loadNamespace(namespaceUri, index);
        }
    }
    /**
     *
     * @param namespaceUri
     * @returns the namespace index of the deleted namespace (-1 if not found)
     */
    deleteNamespace(namespaceUri) {
        var _a;
        const addressSpace = (_a = this.testServer) === null || _a === void 0 ? void 0 : _a.engine.addressSpace;
        if (!addressSpace) {
            return -1;
        }
        /*
            const namespaces = makeNodeId(ObjectIds.Server_Namespaces);
            const namespacesNode = addressSpace.findNode(namespaces) as UAObject;
            const namespaceNode = namespacesNode.getChildByName(namespaceUri)
            if(namespaceNode) {
                addressSpace.deleteNode(namespaceNode);
            }
    */
        const namespace = addressSpace === null || addressSpace === void 0 ? void 0 : addressSpace.getNamespace(namespaceUri);
        if (!namespace) {
            return -1;
        }
        for (const node of Object.values(namespace._nodeid_index)) {
            if (node.addressSpace) {
                try {
                    namespace.deleteNode(node);
                }
                catch (e) {
                    //(node as any).dispose();
                    console.log(e);
                }
            }
        }
        namespace.dispose();
        const index = namespace.index;
        const nsArray = addressSpace.getNamespaceArray();
        nsArray.splice(index, 1);
        return index;
    }
    loadNamespace(namespaceUri, index) {
        var _a;
        const addressSpace = (_a = this.testServer) === null || _a === void 0 ? void 0 : _a.engine.addressSpace; // AddressSpace.create();
        if (!addressSpace) {
            return;
        }
        const loader = new load_nodeset2_1.NodeSetLoader(addressSpace);
        node_opcua_1.readNodeSet2XmlFile(this.nodesetMap.get(namespaceUri), (err, xmlFile) => {
            if (err) {
                console.error(err);
                return;
            }
            if (!xmlFile) {
                return;
            }
            loader.addNodeSet(xmlFile, (err) => {
                if (err) {
                    console.error(err);
                }
                const nsarray = addressSpace.getNamespaceArray();
                const newNamespace = nsarray.pop();
                addressSpace.getNamespaceArray().splice(index, 0, newNamespace);
                newNamespace.index = index;
                addressSpace.suspendBackReference = false;
                const nodes = Object.values(newNamespace._nodeid_index);
                for (const node of nodes) {
                    node.propagate_back_references();
                }
                for (const node of nodes) {
                    node.install_extra_properties();
                }
            });
        });
    }
}
exports.Controller = Controller;
//# sourceMappingURL=controller.js.map