"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.NodeManager = void 0;
const node_opcua_1 = require("node-opcua");
class NodeManager {
    constructor(controlServer, namespace) {
        this.controlServer = controlServer;
        this.namespace = namespace;
        this.init();
    }
    init() {
        var _a, _b;
        const ns = (_a = this.controlServer.engine.addressSpace) === null || _a === void 0 ? void 0 : _a.getNamespace(this.namespace);
        const obj = ns === null || ns === void 0 ? void 0 : ns.addObject({
            organizedBy: (_b = this.controlServer.engine.addressSpace) === null || _b === void 0 ? void 0 : _b.rootFolder.objects,
            browseName: "NodeManager",
            nodeId: node_opcua_1.coerceNodeId("s=NodeManager", this.namespace),
        });
        // start test server method
        const methodAddObject = ns === null || ns === void 0 ? void 0 : ns.addMethod(obj, {
            browseName: "addObject",
            nodeId: node_opcua_1.coerceNodeId("s=NodeManager.addObject", 1),
            inputArguments: [
                { name: "nodeId", dataType: node_opcua_1.DataType.NodeId },
                { name: "browseName", dataType: node_opcua_1.DataType.String },
                { name: "parent", dataType: node_opcua_1.DataType.NodeId },
            ],
        });
        methodAddObject === null || methodAddObject === void 0 ? void 0 : methodAddObject.bindMethod((inputArguments, _context, callback) => __awaiter(this, void 0, void 0, function* () {
            var _c, _d, _e;
            const nodeId = inputArguments[0].value;
            const browseName = inputArguments[1].value;
            const parent = inputArguments[2].value;
            const targetNs = (_d = (_c = this.testServer) === null || _c === void 0 ? void 0 : _c.engine.addressSpace) === null || _d === void 0 ? void 0 : _d.getNamespace((_e = nodeId.namespaceUri) !== null && _e !== void 0 ? _e : nodeId.namespace);
            const obj = targetNs === null || targetNs === void 0 ? void 0 : targetNs.addObject({
                organizedBy: parent,
                browseName,
                nodeId,
            });
            if (obj) {
                callback(null, { statusCode: node_opcua_1.StatusCodes.Good });
            }
            else {
                callback(new Error("failed to create object '" +
                    nodeId +
                    "' on parent '" +
                    parent +
                    "'"), { statusCode: node_opcua_1.StatusCodes.BadInvalidArgument });
            }
        }));
        const methodAddVariable = ns === null || ns === void 0 ? void 0 : ns.addMethod(obj, {
            browseName: "addVariable",
            nodeId: node_opcua_1.coerceNodeId("s=NodeManager.addVariable", 1),
            inputArguments: [
                { name: "nodeId", dataType: node_opcua_1.DataType.NodeId },
                { name: "browseName", dataType: node_opcua_1.DataType.String },
                { name: "parent", dataType: node_opcua_1.DataType.NodeId },
                { name: "value", dataType: node_opcua_1.DataType.Variant },
                {
                    name: "displayName",
                    dataType: node_opcua_1.DataType.LocalizedText,
                    valueRank: -3 /* Scalar or one dimensional array */,
                },
            ],
        });
        methodAddVariable === null || methodAddVariable === void 0 ? void 0 : methodAddVariable.inputArguments;
        methodAddVariable === null || methodAddVariable === void 0 ? void 0 : methodAddVariable.bindMethod((inputArguments, _context, callback) => __awaiter(this, void 0, void 0, function* () {
            var _f, _g, _h;
            const nodeId = inputArguments[0].value;
            const browseName = inputArguments[1].value;
            const parent = inputArguments[2].value;
            const value = inputArguments[3];
            const displayName = Array.isArray(inputArguments[4].value) &&
                inputArguments[4].value.length > 0
                ? inputArguments[4].value
                : undefined;
            const targetNs = (_g = (_f = this.testServer) === null || _f === void 0 ? void 0 : _f.engine.addressSpace) === null || _g === void 0 ? void 0 : _g.getNamespace((_h = nodeId.namespaceUri) !== null && _h !== void 0 ? _h : nodeId.namespace);
            const variable = targetNs === null || targetNs === void 0 ? void 0 : targetNs.addVariable({
                browseName,
                nodeId,
                accessLevel: node_opcua_1.AccessLevelFlag.CurrentWrite | node_opcua_1.AccessLevelFlag.CurrentWrite,
                valueRank: 1,
                value,
                dataType: value.dataType,
                organizedBy: parent,
                displayName,
            });
            if (variable) {
                callback(null, { statusCode: node_opcua_1.StatusCodes.Good });
            }
            else {
                callback(new Error(`failed to create object '${nodeId}' on parent '${parent}' with value '${value}'`), { statusCode: node_opcua_1.StatusCodes.BadInvalidArgument });
            }
        }));
        const methodAddNamespaceForComplianceTesting = ns === null || ns === void 0 ? void 0 : ns.addMethod(obj, {
            browseName: "addComplianceTestNamespace",
            nodeId: node_opcua_1.coerceNodeId("s=NodeManager.addComplianceTestNamespace", 1),
            inputArguments: [],
            outputArguments: [{ name: "namespace", dataType: node_opcua_1.DataType.UInt16 }],
        });
        methodAddNamespaceForComplianceTesting === null || methodAddNamespaceForComplianceTesting === void 0 ? void 0 : methodAddNamespaceForComplianceTesting.bindMethod((_inputArguments, _context, callback) => __awaiter(this, void 0, void 0, function* () {
            var _j, _k;
            if ((_j = this.testServer) === null || _j === void 0 ? void 0 : _j.engine.addressSpace) {
                node_opcua_1.build_address_space_for_conformance_testing((_k = this.testServer) === null || _k === void 0 ? void 0 : _k.engine.addressSpace, undefined);
                const namespace = this.testServer.engine.addressSpace.getNamespaceIndex("urn://node-opcua-simulator");
                callback(null, {
                    statusCode: node_opcua_1.StatusCodes.Good,
                    outputArguments: [
                        new node_opcua_1.Variant({ value: namespace, dataType: node_opcua_1.DataType.UInt16 }),
                    ],
                });
            }
            else {
                callback(new Error("Test Server missing!"), {
                    statusCode: node_opcua_1.StatusCodes.BadInvalidState,
                });
            }
        }));
    }
}
exports.NodeManager = NodeManager;
//# sourceMappingURL=node-manager.js.map