"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.startTestServer = void 0;
const server_endpoint_1 = require("./ws/server-endpoint");
const ws_opcua_server_1 = require("./ws/ws-opcua-server");
const node_opcua_nodesets_1 = require("node-opcua-nodesets");
function adjust_address_space(server) {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        // reduce the minimum sampling interval of CurrentTime from 1000 to 50 to possibly speed up tests
        const currentTimeNode = (_a = server.engine.addressSpace) === null || _a === void 0 ? void 0 : _a.getDefaultNamespace().findNode("i=2258");
        currentTimeNode.minimumSamplingInterval = 50;
    });
}
/**
 * a list of nodeset file names (excluding the standard nodes / namespace)
 * @param nodeSetFileNames
 */
function startTestServer(nodeSetFileNames) {
    return __awaiter(this, void 0, void 0, function* () {
        const server = new ws_opcua_server_1.WsOPCUAServer({
            port: 4841,
            alternateEndpoints: [
                {
                    transportType: server_endpoint_1.TransportType.WEBSOCKET,
                    port: 4445,
                },
            ],
            resourcePath: "",
            buildInfo: {
                productName: "wsopcua-test-server",
                productUri: "wsopcua-test-server",
                buildNumber: "1",
                buildDate: new Date(Date.now()),
            },
            serverInfo: {
                applicationUri: "wsopcua-test-server",
            },
            nodeset_filename: [node_opcua_nodesets_1.nodesets.standard, ...nodeSetFileNames],
            serverCapabilities: {
                minSupportedSampleRate: 10,
            },
        });
        yield server.initialize();
        adjust_address_space(server);
        console.log("test server initialized");
        yield server.start(function (err) {
            if (err) {
                console.log("failed to start test server !");
                console.log(err);
            }
            else {
                console.log("Test Server is now listening ... ( press CTRL+C to stop)");
                console.log("port ", server.endpoints[0].port);
            }
        });
        return server;
    });
}
exports.startTestServer = startTestServer;
//# sourceMappingURL=test-server.js.map