/// <reference types="node" />
import { EventEmitter } from "ws";
import * as net from 'net';
export declare class WebSocketSocketWrapper implements net.Socket {
    write(str: Uint8Array | string, encoding?: string, cb?: (err?: Error) => void): boolean;
    write(str: Uint8Array | string, cb?: (err?: Error) => void): boolean;
    connect(options: net.SocketConnectOpts, connectionListener?: () => void): this;
    connect(port: number, host: string, connectionListener?: () => void): this;
    connect(port: number, connectionListener?: () => void): this;
    connect(path: string, connectionListener?: () => void): this;
    setEncoding(encoding?: string): this;
    pause(): this;
    resume(): this;
    setTimeout(timeout: number, callback?: () => void): this;
    setNoDelay(noDelay?: boolean): this;
    setKeepAlive(enable?: boolean, initialDelay?: number): this;
    address(): string | net.AddressInfo;
    unref(): void;
    ref(): void;
    get bufferSize(): number;
    get bytesWritten(): number;
    get connecting(): boolean;
    get destroyed(): boolean;
    get localAddress(): string;
    get localPort(): number;
    get remoteAddress(): string | undefined;
    get remoteFamily(): string | undefined;
    get remotePort(): number | undefined;
    end(cb?: () => void): void;
    end(buffer: string | Uint8Array, cb?: () => void): void;
    end(str: string | Uint8Array, encoding?: string, cb?: () => void): void;
    addListener(event: string, listener: (...args: any[]) => void): this;
    addListener(event: 'close', listener: (had_error: boolean) => void): this;
    addListener(event: 'connect', listener: () => void): this;
    addListener(event: 'data', listener: (data: Buffer) => void): this;
    addListener(event: 'drain', listener: () => void): this;
    addListener(event: 'end', listener: () => void): this;
    addListener(event: 'error', listener: (err: Error) => void): this;
    addListener(event: 'lookup', listener: (err: Error, address: string, family: string | number, host: string) => void): this;
    addListener(event: 'timeout', listener: () => void): this;
    emit(event: string | symbol, ...args: any[]): boolean;
    on(event: string, listener: (...args: any[]) => void): this;
    on(event: 'close', listener: (had_error: boolean) => void): this;
    on(event: 'connect', listener: () => void): this;
    on(event: 'data', listener: (data: Buffer) => void): this;
    on(event: 'drain', listener: () => void): this;
    on(event: 'end', listener: () => void): this;
    on(event: 'error', listener: (err: Error) => void): this;
    on(event: 'lookup', listener: (err: Error, address: string, family: string | number, host: string) => void): this;
    on(event: 'timeout', listener: () => void): this;
    once(event: string, listener: (...args: any[]) => void): this;
    once(event: 'close', listener: (had_error: boolean) => void): this;
    once(event: 'connect', listener: () => void): this;
    once(event: 'data', listener: (data: Buffer) => void): this;
    once(event: 'drain', listener: () => void): this;
    once(event: 'end', listener: () => void): this;
    once(event: 'error', listener: (err: Error) => void): this;
    once(event: 'lookup', listener: (err: Error, address: string, family: string | number, host: string) => void): this;
    once(event: 'timeout', listener: () => void): this;
    prependListener(event: string, listener: (...args: any[]) => void): this;
    prependListener(event: 'close', listener: (had_error: boolean) => void): this;
    prependListener(event: 'connect', listener: () => void): this;
    prependListener(event: 'data', listener: (data: Buffer) => void): this;
    prependListener(event: 'drain', listener: () => void): this;
    prependListener(event: 'end', listener: () => void): this;
    prependListener(event: 'error', listener: (err: Error) => void): this;
    prependListener(event: 'lookup', listener: (err: Error, address: string, family: string | number, host: string) => void): this;
    prependListener(event: 'timeout', listener: () => void): this;
    prependOnceListener(event: string, listener: (...args: any[]) => void): this;
    prependOnceListener(event: 'close', listener: (had_error: boolean) => void): this;
    prependOnceListener(event: 'connect', listener: () => void): this;
    prependOnceListener(event: 'data', listener: (data: Buffer) => void): this;
    prependOnceListener(event: 'drain', listener: () => void): this;
    prependOnceListener(event: 'end', listener: () => void): this;
    prependOnceListener(event: 'error', listener: (err: Error) => void): this;
    prependOnceListener(event: 'lookup', listener: (err: Error, address: string, family: string | number, host: string) => void): this;
    prependOnceListener(event: 'timeout', listener: () => void): this;
    get writable(): boolean;
    get writableEnded(): boolean;
    get writableFinished(): boolean;
    get writableHighWaterMark(): number;
    get writableLength(): number;
    get writableObjectMode(): boolean;
    get readable(): boolean;
    get readableEncoding(): "ascii" | "utf8" | "utf-8" | "utf16le" | "ucs2" | "ucs-2" | "base64" | "latin1" | "binary" | "hex" | null;
    get readableEnded(): boolean;
    get readableFlowing(): boolean | null;
    get readableHighWaterMark(): number;
    get readableLength(): number;
    get readableObjectMode(): boolean;
    get bytesRead(): number;
    _write(chunk: any, encoding: string, callback: (error?: Error | null) => void): void;
    _destroy(error: Error | null, callback: (error: Error | null) => void): void;
    _final(callback: (error?: Error | null) => void): void;
    setDefaultEncoding(encoding: string): this;
    cork(): void;
    uncork(): void;
    _read(size: number): void;
    read(size?: number): void;
    isPaused(): boolean;
    unpipe(destination?: NodeJS.WritableStream): this;
    unshift(chunk: any, encoding?: BufferEncoding): void;
    wrap(oldStream: NodeJS.ReadableStream): this;
    push(chunk: any, encoding?: string): boolean;
    destroy(error?: Error): void;
    removeListener(event: 'close', listener: () => void): this;
    removeListener(event: 'data', listener: (chunk: any) => void): this;
    removeListener(event: 'end', listener: () => void): this;
    removeListener(event: 'readable', listener: () => void): this;
    removeListener(event: 'error', listener: (err: Error) => void): this;
    removeListener(event: string | symbol, listener: (...args: any[]) => void): this;
    [Symbol.asyncIterator](): AsyncIterableIterator<any>;
    pipe<T extends NodeJS.WritableStream>(destination: T, options?: {
        end?: boolean | undefined;
    }): T;
    off(event: string | symbol, listener: (...args: any[]) => void): this;
    removeAllListeners(event?: string | symbol): this;
    setMaxListeners(n: number): this;
    getMaxListeners(): number;
    listeners(event: string | symbol): Function[];
    rawListeners(event: string | symbol): Function[];
    eventNames(): (string | symbol)[];
    listenerCount(type: string | symbol): number;
    webSocket: WebSocket & EventEmitter & {
        _socket: net.Socket;
    };
    constructor(websocket: WebSocket);
}
