"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.WsOPCUAServer = void 0;
const node_opcua_1 = require("node-opcua");
const server_endpoint_1 = require("./server-endpoint");
// @ts-ignore
class WsOPCUAServer extends node_opcua_1.OPCUAServer {
    constructor(options) {
        super(options);
    }
    getTransportConstructor(transportType) {
        switch (transportType) {
            case server_endpoint_1.TransportType.TCP:
                return node_opcua_1.OPCUAServerEndPoint;
            case server_endpoint_1.TransportType.WEBSOCKET:
                return server_endpoint_1.OPCUAWsServerEndPoint;
            case server_endpoint_1.TransportType.WEBSOCKET_SECURE:
                return server_endpoint_1.OPCUAWsSecureServerEndPoint;
        }
    }
    createEndpoint(port1, transportType, serverOptions) {
        // add the tcp/ip endpoint with no security, a ws endpoint or a wss endpoint
        let transportConstructor = this.getTransportConstructor(transportType);
        const endPoint = new transportConstructor({
            port: port1,
            certificateManager: this.serverCertificateManager,
            certificateChain: this.getCertificateChain(),
            privateKey: this.getPrivateKey(),
            defaultSecureTokenLifetime: serverOptions.defaultSecureTokenLifetime || 600000,
            timeout: serverOptions.timeout || 3 * 60 * 1000,
            maxConnections: this.maxConnectionsPerEndpoint,
            // @ts-ignore
            objectFactory: this.objectFactory,
            serverInfo: this.serverInfo,
        });
        return endPoint;
    }
    createEndpointDescriptions(serverOption, endpointOptions) {
        /* istanbul ignore next */
        if (!endpointOptions) {
            throw new Error("internal error");
        }
        var hostname = node_opcua_1.getFullyQualifiedDomainName();
        endpointOptions.hostname = endpointOptions.hostname || hostname;
        endpointOptions.port = endpointOptions.port || 26543;
        /* istanbul ignore next */
        if (!endpointOptions.hasOwnProperty("port") ||
            !isFinite(endpointOptions.port) ||
            typeof endpointOptions.port !== "number") {
            throw new Error("expecting a valid port (number)");
        }
        const port = Number(endpointOptions.port || 0);
        endpointOptions.transportType =
            endpointOptions.transportType || server_endpoint_1.TransportType.TCP;
        const endPoint = this.createEndpoint(port, endpointOptions.transportType, serverOption);
        endpointOptions.alternateHostname = endpointOptions.alternateHostname || [];
        const alternateHostname = endpointOptions.alternateHostname instanceof Array
            ? endpointOptions.alternateHostname
            : [endpointOptions.alternateHostname];
        const allowAnonymous = endpointOptions.allowAnonymous === undefined
            ? true
            : !!endpointOptions.allowAnonymous;
        endPoint.addStandardEndpointDescriptions({
            allowAnonymous,
            securityModes: endpointOptions.securityModes,
            securityPolicies: endpointOptions.securityPolicies,
            hostname: endpointOptions.hostname,
            alternateHostname,
            disableDiscovery: !!endpointOptions.disableDiscovery,
            // xx                hostname,
            resourcePath: serverOption.resourcePath || "",
        });
        return endPoint;
    }
}
exports.WsOPCUAServer = WsOPCUAServer;
//# sourceMappingURL=ws-opcua-server.js.map